"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createApiClients = exports.createApiClient = void 0;
var tslib_1 = require("tslib");
var promises_1 = tslib_1.__importDefault(require("fs/promises"));
var fs_extra_1 = require("fs-extra");
var path_1 = require("path");
var node_fetch_1 = tslib_1.__importDefault(require("node-fetch"));
var codegen_1 = require("farrow-api/dist/codegen");
var writeFile = function (filename, content) { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
    return tslib_1.__generator(this, function (_a) {
        switch (_a.label) {
            case 0:
                _a.trys.push([0, , 2, 4]);
                return [4 /*yield*/, (0, fs_extra_1.ensureDir)((0, path_1.dirname)(filename))];
            case 1:
                _a.sent();
                return [3 /*break*/, 4];
            case 2: return [4 /*yield*/, promises_1.default.writeFile(filename, content)];
            case 3:
                _a.sent();
                return [7 /*endfinally*/];
            case 4: return [2 /*return*/];
        }
    });
}); };
var createApiClient = function (options) {
    var config = tslib_1.__assign(tslib_1.__assign({}, options), { pollingInterval: 3000, introspectionUrl: '/__introspection__', getIntrospection: function (x) { return x; } });
    var tid = null;
    var prevText = '';
    var getIntrospection = function () { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
        var url, response, body, introspection, text, error_1;
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    url = config.introspectionUrl.startsWith('http')
                        ? config.introspectionUrl
                        : config.src + config.introspectionUrl;
                    _a.label = 1;
                case 1:
                    _a.trys.push([1, 4, , 5]);
                    return [4 /*yield*/, (0, node_fetch_1.default)(url)];
                case 2:
                    response = _a.sent();
                    return [4 /*yield*/, response.json()];
                case 3:
                    body = _a.sent();
                    introspection = config.getIntrospection(body);
                    text = JSON.stringify(introspection);
                    if (text === prevText) {
                        return [2 /*return*/];
                    }
                    prevText = text;
                    return [2 /*return*/, introspection];
                case 4:
                    error_1 = _a.sent();
                    if (error_1 instanceof Error) {
                        console.log("Failed to fetch ".concat(config.src, "\nerror: ").concat(error_1.message));
                        return [2 /*return*/];
                    }
                    console.log("Failed to fetch ".concat(config.src));
                    return [3 /*break*/, 5];
                case 5: return [2 /*return*/];
            }
        });
    }); };
    var sync = function () { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
        var result, source;
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, getIntrospection()];
                case 1:
                    result = _a.sent();
                    if (!result) {
                        return [2 /*return*/];
                    }
                    source = (0, codegen_1.codegen)(result, tslib_1.__assign({}, config.codegen));
                    if (typeof config.logger === 'function') {
                        config.logger(options);
                    }
                    else if (config.logger !== false) {
                        console.log("synced farrow-api", {
                            src: config.src,
                            dist: config.dist,
                        });
                    }
                    return [4 /*yield*/, writeFile(config.dist, source)];
                case 2:
                    _a.sent();
                    return [2 /*return*/];
            }
        });
    }); };
    var start = function () {
        stop();
        tid = setInterval(function () { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
            var error_2;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        _a.trys.push([0, 2, , 3]);
                        return [4 /*yield*/, sync()];
                    case 1:
                        _a.sent();
                        return [3 /*break*/, 3];
                    case 2:
                        error_2 = _a.sent();
                        console.log(error_2.stack);
                        return [3 /*break*/, 3];
                    case 3: return [2 /*return*/];
                }
            });
        }); }, config.pollingInterval);
    };
    var stop = function () {
        if (tid !== null) {
            clearInterval(tid);
        }
    };
    return {
        sync: sync,
        start: start,
        stop: stop,
    };
};
exports.createApiClient = createApiClient;
var createApiClients = function (options) {
    var clients = options.services.map(exports.createApiClient);
    var sync = function () { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
        var promises;
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    promises = clients.map(function (client) { return client.sync(); });
                    return [4 /*yield*/, Promise.all(promises)];
                case 1:
                    _a.sent();
                    return [2 /*return*/];
            }
        });
    }); };
    var start = function () {
        clients.map(function (client) { return client.start(); });
    };
    var stop = function () {
        clients.map(function (syncer) { return syncer.stop(); });
    };
    return {
        sync: sync,
        start: start,
        stop: stop,
    };
};
exports.createApiClients = createApiClients;
//# sourceMappingURL=index.js.map