"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var commander_1 = require("commander");
var chalk_1 = tslib_1.__importDefault(require("chalk"));
var leven_1 = tslib_1.__importDefault(require("leven"));
var dev_1 = tslib_1.__importDefault(require("../scripts/dev"));
var build_1 = tslib_1.__importDefault(require("../scripts/build"));
var start_1 = tslib_1.__importDefault(require("../scripts/start"));
process.on('unhandledRejection', function (err) {
    throw err;
});
var pkg = require('../../package.json');
var program = new commander_1.Command();
program.version(pkg.version).name(pkg.name);
program
    .command('dev')
    .description("start development mode at ".concat(process.cwd()))
    .option('-c, --config <config>', 'config file path')
    .action(function (options) {
    return (0, dev_1.default)(options);
});
program
    .command('build')
    .description("bundle project at ".concat(process.cwd()))
    .option('-c, --config <config>', 'config file path')
    .action(function (options) {
    return (0, build_1.default)(options);
});
program
    .command('start')
    .description("start production mode at ".concat(process.cwd()))
    .option('-c, --config <config>', 'config file path')
    .action(function (options) {
    return (0, start_1.default)(options);
});
// output help information on unknown COMMANDS
program.arguments('<command>').action(function (cmd) {
    program.outputHelp();
    console.log("  ".concat(chalk_1.default.red("Unknown command ".concat(chalk_1.default.yellow(cmd), "."))));
    console.log();
    suggestCommands(cmd);
});
program.on('--help', function () {
    console.log('');
    console.log('Example call:');
    console.log("  $ ".concat(pkg.name, " --help"));
});
program.parse(process.argv);
if (!process.argv.slice(2).length) {
    program.outputHelp();
}
function suggestCommands(unknownCommand) {
    // eslint-disable-next-line @typescript-eslint/ban-ts-comment
    // @ts-ignore
    var availableCommands = program.commands.map(function (cmd) { return cmd._name; });
    var suggestion = undefined;
    availableCommands.forEach(function (cmd) {
        var isBestMatch = (0, leven_1.default)(cmd, unknownCommand) < (0, leven_1.default)(suggestion || '', unknownCommand);
        if ((0, leven_1.default)(cmd, unknownCommand) < 3 && isBestMatch) {
            suggestion = cmd;
        }
    });
    if (suggestion) {
        console.log("  ".concat(chalk_1.default.red("Did you mean ".concat(chalk_1.default.yellow(suggestion), "?"))));
    }
}
//# sourceMappingURL=index.js.map