/// <reference types="node" />
import { BuildOptions } from 'esbuild';
import * as esbuild from 'esbuild';
export type BundlerOptions = {
    /**
     * auto add closest package.json dependenties to esbuild external or not
     */
    autoAddExternal?: boolean;
    build: BuildOptions;
};
export declare const createBundler: (options: BundlerOptions) => {
    build: () => Promise<esbuild.BuildIncremental>;
    dispose: () => Promise<void>;
};
export type ServerBundlerOptions = {
    /**
     * filename of entry
     */
    entry?: string;
    /**
     * folder of source code
     */
    src?: string;
    /**
     * folder of output code
     */
    dist?: string;
    /**
     * - args for node.js
     * - eg. ['--inspect-brk'] for debugging
     */
    nodeArgs?: string[];
    /**
     * - env for node.js
     * - eg. { NODE_ENV: 'production' }
     * - NODE_ENV = production in `farrow start`
     * - NODE_ENV = development in `farrow dev`
     */
    env?: NodeJS.ProcessEnv;
    /**
     * other options for esbuild
     */
    esbuild?: Omit<BuildOptions, 'entryPoints' | 'outdir' | 'outbase'>;
    /**
     * alias of autoExternal
     * auto add closest package.json dependenties to esbuild external or not
     */
    autoAddExternal?: boolean;
    /**
     * auto add closest package.json dependenties to esbuild external or not
     */
    autoExternal?: boolean;
};
export type StartOptions = {
    build?: boolean;
    watch?: boolean;
    run?: boolean;
};
export declare const createServerBundler: (options?: ServerBundlerOptions) => {
    start: (startOptions?: StartOptions) => Promise<void>;
    stop: () => Promise<void>;
};
export type ServerBundlersOptions = {
    bundlers: ServerBundlerOptions[];
};
export declare const createServerBundlers: (options: ServerBundlersOptions) => {
    start: (startOptions?: StartOptions) => Promise<void>;
    stop: () => Promise<void>;
};
