"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createServerBundlers = exports.createServerBundler = exports.createBundler = void 0;
var tslib_1 = require("tslib");
var path_1 = tslib_1.__importDefault(require("path"));
var execa_1 = tslib_1.__importDefault(require("execa"));
var esbuild = tslib_1.__importStar(require("esbuild"));
var slash_1 = tslib_1.__importDefault(require("slash"));
var promises_1 = tslib_1.__importDefault(require("fs/promises"));
var read_pkg_up_1 = tslib_1.__importDefault(require("read-pkg-up"));
var esbuild_node_externals_1 = require("esbuild-node-externals");
var watchFiles_1 = require("../util/watchFiles");
var join_1 = require("../util/join");
var memo_1 = require("../util/memo");
var mergeList = function () {
    var _a;
    var args = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        args[_i] = arguments[_i];
    }
    var list = args.filter(Boolean);
    return (_a = []).concat.apply(_a, tslib_1.__spreadArray([], tslib_1.__read(list), false));
};
var safeGetKeys = function (input) {
    if (!input)
        return [];
    return Object.keys(input);
};
var createBundler = function (options) {
    var config = tslib_1.__assign({ autoAddExternal: true }, options);
    var getBuilder = (0, memo_1.memo)(function () { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
        var pkgResult, external, plugins, result;
        var _a, _b, _c, _d, _e, _f, _g;
        return tslib_1.__generator(this, function (_h) {
            switch (_h.label) {
                case 0: return [4 /*yield*/, (0, read_pkg_up_1.default)({
                        cwd: (_a = config.build.entryPoints) === null || _a === void 0 ? void 0 : _a[0],
                    })];
                case 1:
                    pkgResult = _h.sent();
                    external = config.autoAddExternal
                        ? mergeList(config.build.external, safeGetKeys((_b = pkgResult === null || pkgResult === void 0 ? void 0 : pkgResult.packageJson) === null || _b === void 0 ? void 0 : _b.devDependencies), safeGetKeys((_c = pkgResult === null || pkgResult === void 0 ? void 0 : pkgResult.packageJson) === null || _c === void 0 ? void 0 : _c.dependencies), safeGetKeys((_d = pkgResult === null || pkgResult === void 0 ? void 0 : pkgResult.packageJson) === null || _d === void 0 ? void 0 : _d.peerDependencies), safeGetKeys((_e = pkgResult === null || pkgResult === void 0 ? void 0 : pkgResult.packageJson) === null || _e === void 0 ? void 0 : _e.optionalDependencies))
                        : config.build.external;
                    plugins = config.autoAddExternal
                        ? tslib_1.__spreadArray(tslib_1.__spreadArray([], tslib_1.__read(((_f = config.build.plugins) !== null && _f !== void 0 ? _f : [])), false), [(0, esbuild_node_externals_1.nodeExternalsPlugin)()], false) : (_g = config.build.plugins) !== null && _g !== void 0 ? _g : [];
                    return [4 /*yield*/, esbuild.build(tslib_1.__assign(tslib_1.__assign({}, config.build), { external: external, plugins: plugins, incremental: true }))];
                case 2:
                    result = _h.sent();
                    return [2 /*return*/, result.rebuild];
            }
        });
    }); });
    var hasBuilt = false;
    var build = function () { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
        var builder;
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    hasBuilt = true;
                    return [4 /*yield*/, getBuilder()];
                case 1:
                    builder = _a.sent();
                    return [2 /*return*/, builder()];
            }
        });
    }); };
    var dispose = function () { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
        var builder;
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    if (!hasBuilt) return [3 /*break*/, 2];
                    return [4 /*yield*/, getBuilder()];
                case 1:
                    builder = _a.sent();
                    builder.dispose();
                    _a.label = 2;
                case 2: return [2 /*return*/];
            }
        });
    }); };
    return {
        build: build,
        dispose: dispose,
    };
};
exports.createBundler = createBundler;
var createServerBundler = function (options) {
    var _a, _b, _c;
    if (options === void 0) { options = {}; }
    var config = tslib_1.__assign(tslib_1.__assign({ entry: 'index.ts', src: 'src', dist: 'dist', nodeArgs: [] }, options), { esbuild: tslib_1.__assign({ sourcemap: true, keepNames: true }, options.esbuild) });
    var srcEntry = (0, join_1.join)(config.src, config.entry);
    var distEntry = (0, join_1.join)(config.dist, config.entry).replace(/\.(tsx?)$/, '.js');
    var bundler = (0, exports.createBundler)({
        autoAddExternal: (_b = (_a = config.autoExternal) !== null && _a !== void 0 ? _a : config.autoAddExternal) !== null && _b !== void 0 ? _b : true,
        build: tslib_1.__assign(tslib_1.__assign({}, config.esbuild), { platform: 'node', bundle: true, entryPoints: [srcEntry], outbase: config.src, outdir: config.dist, plugins: tslib_1.__spreadArray(tslib_1.__spreadArray([], tslib_1.__read(((_c = config.esbuild.plugins) !== null && _c !== void 0 ? _c : [])), false), [createResolveDirnamePlugin(config.dist)], false) }),
    });
    var childProcess;
    var run = function () {
        cancel();
        childProcess = (0, execa_1.default)('node', tslib_1.__spreadArray(tslib_1.__spreadArray([], tslib_1.__read(config.nodeArgs), false), [distEntry], false), {
            stdout: 'inherit',
            stderr: 'inherit',
            env: tslib_1.__assign({ NODE_ENV: process.env.NODE_ENV }, config.env),
        });
        childProcess.catch(function (error) {
            if (error.isCanceled)
                return;
            throw error;
        });
    };
    var cancel = function () {
        if (childProcess) {
            childProcess.cancel();
            childProcess = null;
        }
    };
    var hasWatcher = false;
    var getWatcher = (0, memo_1.memo)(function () { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
        var watcher;
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    hasWatcher = true;
                    return [4 /*yield*/, (0, watchFiles_1.watchFiles)('.', {
                            cwd: config.src,
                            usePolling: true,
                        })];
                case 1:
                    watcher = _a.sent();
                    watcher.on('all', function () { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
                        var error_1;
                        return tslib_1.__generator(this, function (_a) {
                            switch (_a.label) {
                                case 0:
                                    _a.trys.push([0, 2, , 3]);
                                    return [4 /*yield*/, build()];
                                case 1:
                                    _a.sent();
                                    run();
                                    return [3 /*break*/, 3];
                                case 2:
                                    error_1 = _a.sent();
                                    console.log('watcher:', error_1.stack);
                                    return [3 /*break*/, 3];
                                case 3: return [2 /*return*/];
                            }
                        });
                    }); });
                    return [2 /*return*/, watcher];
            }
        });
    }); });
    var watch = function () { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, getWatcher()];
                case 1:
                    _a.sent();
                    return [2 /*return*/];
            }
        });
    }); };
    var build = function () { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
        var start;
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    start = Date.now();
                    console.log('start build');
                    return [4 /*yield*/, bundler.build()];
                case 1:
                    _a.sent();
                    console.log("finish build in ".concat((Date.now() - start).toFixed(2), "ms"));
                    return [2 /*return*/];
            }
        });
    }); };
    var isStarted = false;
    var start = function (startOptions) { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
        var startConfig;
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    if (!isStarted) return [3 /*break*/, 2];
                    return [4 /*yield*/, stop()];
                case 1:
                    _a.sent();
                    _a.label = 2;
                case 2:
                    // eslint-disable-next-line require-atomic-updates
                    isStarted = true;
                    startConfig = tslib_1.__assign({}, startOptions);
                    if (!startConfig.build) return [3 /*break*/, 4];
                    return [4 /*yield*/, build()];
                case 3:
                    _a.sent();
                    _a.label = 4;
                case 4:
                    if (!startConfig.watch) return [3 /*break*/, 6];
                    return [4 /*yield*/, watch()];
                case 5:
                    _a.sent();
                    _a.label = 6;
                case 6:
                    if (startConfig.run) {
                        run();
                    }
                    if (!(!startConfig.run && !startConfig.watch)) return [3 /*break*/, 8];
                    return [4 /*yield*/, stop()];
                case 7:
                    _a.sent();
                    _a.label = 8;
                case 8: return [2 /*return*/];
            }
        });
    }); };
    var stop = function () { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
        var watcher;
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    cancel();
                    return [4 /*yield*/, bundler.dispose()];
                case 1:
                    _a.sent();
                    if (!hasWatcher) return [3 /*break*/, 4];
                    return [4 /*yield*/, getWatcher()];
                case 2:
                    watcher = _a.sent();
                    return [4 /*yield*/, watcher.close()];
                case 3:
                    _a.sent();
                    _a.label = 4;
                case 4:
                    isStarted = false;
                    return [2 /*return*/];
            }
        });
    }); };
    return {
        start: start,
        stop: stop,
    };
};
exports.createServerBundler = createServerBundler;
var createServerBundlers = function (options) {
    var bundlers = options.bundlers.map(exports.createServerBundler);
    var start = function (startOptions) { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
        var promises;
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    promises = bundlers.map(function (bundler) { return bundler.start(startOptions); });
                    return [4 /*yield*/, Promise.all(promises)];
                case 1:
                    _a.sent();
                    return [2 /*return*/];
            }
        });
    }); };
    var stop = function () { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
        var promises;
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    promises = bundlers.map(function (bundler) { return bundler.stop(); });
                    return [4 /*yield*/, Promise.all(promises)];
                case 1:
                    _a.sent();
                    return [2 /*return*/];
            }
        });
    }); };
    return {
        start: start,
        stop: stop,
    };
};
exports.createServerBundlers = createServerBundlers;
var createResolveDirnamePlugin = function (dist) {
    return {
        name: 'farrow.resolve.dirname',
        setup: function (build) {
            build.onLoad({
                filter: /.*\.(j|t)sx?$/,
            }, function (args) { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
                var content, dirname, relative, extname;
                return tslib_1.__generator(this, function (_a) {
                    switch (_a.label) {
                        case 0: return [4 /*yield*/, promises_1.default.readFile(args.path)];
                        case 1:
                            content = (_a.sent()).toString();
                            dirname = path_1.default.dirname(args.path);
                            relative = path_1.default.relative(dist, dirname);
                            extname = path_1.default.extname(args.path);
                            return [2 /*return*/, {
                                    contents: content.replace(/__dirname/g, "require('path').join(__dirname, \"".concat((0, slash_1.default)(relative), "\")")),
                                    loader: extname.slice(1),
                                }];
                    }
                });
            }); });
        },
    };
};
//# sourceMappingURL=server.js.map